package
{
	/**
	 * 
	 * original by DarceyLloyd (Darcey)
	 * https://github.com/DarceyLloyd
		
		http://www.allforthecode.co.uk/
		
	 * found here 
	 * https://github.com/away3d/away3d-core-fp11/issues/152
	 * 
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	import away3d.cameras.Camera3D;
	import away3d.containers.ObjectContainer3D;
	import away3d.containers.View3D;
	import away3d.controllers.HoverController;
	import away3d.debug.AwayStats;
	import away3d.entities.Mesh;
	import away3d.lights.PointLight;
	import away3d.materials.BitmapMaterial;
	import away3d.materials.ColorMaterial;
	import away3d.primitives.Sphere;
	import flash.display.BlendMode;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageDisplayState;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.utils.Dictionary;
	
	public class BaseScene extends Sprite
	{
		//debug
		public var debug:Boolean = false;
		public var statistics:Boolean = false;
		
		//default materials
		[Embed(source="res/textures/damier64_64.png")]
		private var uvMap:Class;
		public var RED:ColorMaterial = new ColorMaterial( 0xFF0000 );
		public var YELLOW:ColorMaterial = new ColorMaterial( 0xFFCC00 );
		public var GREEN:ColorMaterial = new ColorMaterial( 0x00cC33 );
		public var BLUE:ColorMaterial = new ColorMaterial( 0x0066DD );
		public var WHITE:ColorMaterial = new ColorMaterial( 0xFFFFFF );
		public var PURPLE:ColorMaterial = new ColorMaterial( 0xCC00CC );
		public var UV_DEBUG:BitmapMaterial = new BitmapMaterial( new uvMap().bitmapData );
		
		//engine variables
		protected var camera:Camera3D;
		protected var view:View3D;
		protected var container:ObjectContainer3D;
		protected var hoverController:HoverController;
		
		//navigation variables
		private var move:Boolean = false;
		private var lastPanAngle:Number;
		private var lastTiltAngle:Number;
		private var lastMouseX:Number;
		private var lastMouseY:Number;
		private var stats:AwayStats;
		protected var cameraDistance:Number = -500;
		
		//lights
		protected var lights:Array = [];
		protected var lightColors:Array = [ 0x537967, 0x41544B, 0x12452D, 0xA2CEB9, 0xB2CEC1 ];
		protected var lightsDistance:Number = 10000;
		
		//materials
		protected var materials:Dictionary = new Dictionary( true );
		
		
		/**
		 * Constructor
		 */
		public function BaseScene()
		{
			if ( stage ) init();
			else addEventListener( Event.ADDED_TO_STAGE, init );
		}
		
		/**
		 * Global initialise function
		 */
		private function init(e:Event = null):void
		{
			if( debug ) _trace("init()");
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			initEngine();
			
		}
		
		/**
		 * Initialise the engine
		 */
		private function initEngine():void
		{
			if( debug ) _trace("initEngine()");
			
			//setup camera for optimal shadow rendering
			camera = new Camera3D();
			camera.lens.far = 10000;
			
			//setup view
			view = new View3D();
			view.backgroundColor = 0;
			view.camera = camera;
			addChild(view);
			
			//main container
			container = new ObjectContainer3D();
			view.scene.addChild( container );
			
			//setup controller to be used on the camera
			hoverController = new HoverController( camera, null, 0, 0, cameraDistance );
			
		}
		
		
		protected function setup():void
		{
			
			if ( debug ) _trace( '------setup-------' );
			
			initLights();
			
			initMaterials();
			
			initObjects();
			
			initListeners();
			
			//add stats
			if ( statistics )
			{
				stats = new AwayStats(view);
				stats.x = stage.stageWidth - stats.width;
				addChild( stats );
			}
			
			if ( debug ) _trace( '----- / setup-----' );
			
		}
		
		/**
		 * Initialise the scene materials
		 */
		protected function initMaterials():void
		{
			if( debug ) _trace("initMaterials()");
			
		}
		
		/**
		 * Initialise the scene lights
		 */
		protected function initLights():void
		{
			if( debug ) _trace("initLights()");
			lights = [];
			var positions:Vector.<Number> = Vector.<Number>( [ -0.798892958090,-0.274518427439,0.535172563978,0.337674633600,-0.783974919468,-0.520921458490,0.717804165557,0.325344236568,0.615555282682,-0.256585841067,0.733149110340,-0.629806388170 ] );
			if ( lightColors.length == 0 ) lightColors = [ 0xFFFFFF ];
			for ( var i:int = 0; i < positions.length; i+=3 )
			{
				
				var light:PointLight = new PointLight();
				light.x = positions[ i ] * lightsDistance;
				light.y = positions[ i + 1 ] * lightsDistance;
				light.z = positions[ i + 2 ] * lightsDistance;
				
				light.color = lightColors[ int( i / 3 ) % lightColors.length ];
				
				light.fallOff = lightsDistance * 1.5;
				container.addChild( light );
				if ( debug ) 
				{
					var s:Sphere = new Sphere( new ColorMaterial( light.color ), 5  );
					s.position = light.position;
					container.addChild( s );
				}
				lights.push( light );
				
			}
		}
		
		/**
		 * Initialise the scene objects
		 */
		protected function initObjects():void
		{
			if ( debug ) 
			{
				_trace("initObjects()");
			}
		}
		
		/**
		 * shortcut to add meshes to scene
		 * @param	mesh 
		 */
		protected function addMesh( ...args ):void 
		{
			for each( var p:* in args ) container.addChild( p );
		}
		/**
		 * shortcut to remove meshes from scene
		 * @param	mesh 
		 */
		protected function removeMesh( mesh:Mesh ):void 
		{
			container.removeChild( mesh );
		}
		
		/**
		 * Initialise the listeners
		 */
		private function initListeners():void
		{
			
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel );
			stage.addEventListener(Event.RESIZE, onResize);
			onResize();
			stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
			
			stage.addEventListener( KeyboardEvent.KEY_DOWN, onKeyDown );
			stage.addEventListener( KeyboardEvent.KEY_UP, onKeyUp );
			
		}
		/**
		 * removes all listeners
		 */
		protected function removeListeners():void
		{
			
			stage.removeEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			stage.removeEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel );
			stage.removeEventListener(Event.RESIZE, onResize);
			stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.removeEventListener( KeyboardEvent.KEY_DOWN, onKeyDown );
			stage.removeEventListener( KeyboardEvent.KEY_UP, onKeyUp );
			
		}
		
		protected function onKeyDown( e:KeyboardEvent ):void
		{
			//loops background colors
			if ( e.charCode == "b".charCodeAt( 0 ) )
			{
				var color:uint;
				switch( view.backgroundColor )
				{
					case 0:				color = 0x808080; 	break;
					case 0x808080:		color = 0xFFFFFF; 	break;
					case 0xFFFFFF:		color = 0; 			break;
				}
				view.backgroundColor = color;
			}
			if ( e.charCode == "f".charCodeAt( 0 ) )
			{
				if ( stage.displayState == StageDisplayState.NORMAL) 
				{
					stage.displayState=StageDisplayState.FULL_SCREEN;
				}
				else 
				{
					stage.displayState=StageDisplayState.NORMAL;
				}
				onResize( null );
			}
		}
		
		protected function onKeyUp( e:KeyboardEvent ):void
		{
			
		}
		/**
		 * Navigation and render loop
		 */
		protected function onEnterFrame(event:Event):void
		{
			if (move)
			{ 
				hoverController.panAngle = 0.3 * (stage.mouseX - lastMouseX) + lastPanAngle;
				hoverController.tiltAngle = 0.3 * (stage.mouseY - lastMouseY) + lastTiltAngle;
			}
			hoverController.distance += ( cameraDistance - hoverController.distance ) * .3;
			hoverController.update()
			view.render();
			
		}
		
		/**
		 * Mouse down listener for navigation
		 */
		private function onMouseDown(event:MouseEvent):void
		{
			lastPanAngle = hoverController.panAngle;
			lastTiltAngle = hoverController.tiltAngle;
			lastMouseX = stage.mouseX;
			lastMouseY = stage.mouseY;
			move = true;
			stage.addEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);
		}
		
		/**
		 * Mouse up listener for navigation
		 */
		private function onMouseUp(event:MouseEvent):void
		{
			move = false;
			stage.removeEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);
		}
		
		/**
		 * Mouse up listener for distance
		 */
		private function onMouseWheel(e:MouseEvent):void 
		{
			cameraDistance += e.delta * 10;
		}
		
		/**
		 * Mouse stage leave listener for navigation
		 */
		private function onStageMouseLeave(event:Event):void
		{
			move = false;
			stage.removeEventListener(Event.MOUSE_LEAVE, onStageMouseLeave);
		}
		
		/**
		 * stage listener for resize events
		 */
		private function onResize(event:Event = null):void
		{
			view.width = stage.stageWidth;
			view.height = stage.stageHeight;
			if ( statistics && stats != null )
			{
				stats.x = stage.stageWidth - stats.width;
			}
		}
	}
}