package triga.spline.utils
{
	import flash.geom.Vector3D;
	public class SimplifiyDouglasPeucker
	{
		 //adapted from http://geometryalgorithms.com/Archive/algorithm_0205/algorithm_0205.htm
		//original copyright message follows
		
		// Copyright 2002, softSurfer (www.softsurfer.com)
		// This code may be freely used and modified for any purpose
		// providing that this copyright notice is included with it.
		// SoftSurfer makes no warranty for this code, and cannot be held
		// liable for any real or imagined damage resulting from its use.
		// Users of this code must verify correctness for their application.
				
		/**
		 * This method will reduce a 2D complex polyline.
		 * @param tol the tolerance of the reduction algorithm. Higher numbers will simplify the line more.
		 * @param V the array of Tuple2fs to be simplified
		 * @return an array of Tuple2f representing the simplified polyline
		 */

		public static function compute( vertices:Vector.<Vector3D>, tol:Number ):Vector.<Vector3D>
		{
			
			var n:int = vertices.length;
			var i:int, b:int, k:int, m:int, pv:int = 0;
			var tol2:Number = tol * tol;
			
			var tmp:Vector.<Vector3D> = new Vector.<Vector3D>( n );
			var marked:Vector.<int> = new Vector.<int>( n );
			
			for ( b = 0; b < n; b++)
			{
				marked[b] = 0;
			}

			//STAGE 1 simple vertex reduction
			tmp[0] = vertices[0];

			for (i = k = 1, pv = 0; i < n; i++)
			{
				if ( distanceSquared( vertices[ i ], vertices[ pv ] ) < tol2) continue;
				
				tmp[k++] = vertices[i];
				
				pv = i;
				
			}
				
			//adding last Vertex
			if (pv < n - 1) tmp[k++] = vertices[n - 1];
			
				
			//STAGE 2 Douglas-Peucker polyline simplify
			//mark the first and last vertices
			marked[0] = marked[k - 1] = 1;
			
			simplifyDP3D(tol, tmp, 0, k - 1, marked);
			
			//copy marked vertices to output
			var out:Vector.<Vector3D> = new Vector.<Vector3D>();
			for (i = m = 0; i < k; i++) 
			{
				
				if (marked[i] == 1) out.push(tmp[i]);
				
			}
			
			trace( out[ 0 ],out[ 0 ].x,out[ 0 ].y );
			
			return out.concat();
			
		}


		private static function simplifyDP3D( tol:Number, v:Vector.<Vector3D>, j:int, k:int, mk:Vector.<int>):void
		{

			if (k <= j+1) return;  //nothing to simplify

			var maxi:int = j;
			var maxd2:Number = 0;
			var tol2:Number = tol * tol;
			
			var u:Vector3D = v[ k ].subtract(v[j]);
			
			var cu:Number = u.dotProduct( u );

			var w:Vector3D, Pb:Vector3D;
			var b:Number = 0, cw:Number= 0, dv2:Number = 0;
			var i:int;
			for ( i = j + 1; i < k; i++ )
			{
				w = v[i].subtract(v[j]);
				cw = w.dotProduct( u );
				
				if (cw <= 0)
				{
					dv2 = distanceSquared(v[i],v[j]);
				}
				else if (cu <= cw)
				{
					dv2 = distanceSquared(v[i],v[k]);
				}
				else
				{
					b = cw / cu;
					u.scaleBy( -b);
					Pb= v[j].subtract( u );
					dv2 = distanceSquared(v[i], Pb);
				}

				if (dv2 <= maxd2) continue;
				maxi = i;
				maxd2 = dv2;
				
			}
			
			if (maxd2 > tol2)
			{
				
				mk[maxi] = 1;
				simplifyDP3D(tol, v, j, maxi, mk);
				simplifyDP3D(tol, v, maxi, k, mk);
				
			}
			return;
		}
		
		static private function distanceSquared( a:Vector3D, b:Vector3D ):Number
		{
			return ( ( a.x - b.x ) * ( a.x - b.x ) + ( a.y - b.y ) * ( a.y - b.y ) + ( a.z - b.z ) * ( a.z - b.z ) );
		}
		
	} 
}