package geometry 
{
	import com.bit101.components.CheckBox;
	import com.bit101.components.PushButton;
	import com.bit101.components.RadioButton;
	import com.bit101.components.Window;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class DrawPanel extends Sprite
	{
		
		static public const CHANGE:String = "change";
		static public const UPDATE:String = "update";
		
		private var canvas:Sprite;
		private var tempCanvas:Shape;
		private var snap:CheckBox;
		private var create:RadioButton;
		private var erase:RadioButton;
		private var move:RadioButton;
		private var insert:RadioButton;
		private var updateButton:PushButton;
		
		private var _gridSize:Number = 10;
		private var _width:Number = 500;
		private var _height:Number = 500;
		private var _selected:Point;
		
		private var _points:Vector.<Point>;
		
		public function DrawPanel( title:String = "canvas", width:Number = 500, height:Number = 500, gridSize:uint = 10 ) 
		{
			_width = width;
			_height = height;
			_gridSize = gridSize;
			
			
			canvas = new Sprite();
			canvas.x = _width * .5;
			canvas.y = _height * .5;
			tempCanvas = new Shape();
			canvas.addChild( tempCanvas );
			
			var canvasWindow:Window = new Window( this, 0, 0, title );
			canvasWindow.width = _width;
			canvasWindow.height = _height;
			addChild( canvasWindow );
			canvasWindow.addChild( canvas );
			var oy:int = - _height *.5 + 10
			snap = new CheckBox( canvas,  - _width *.5 + 10, oy, "snap to grid", mouseHandler );
			
			create = new RadioButton( canvas, snap.x + snap.width + 20, oy, "create", true, mouseHandler);
			erase = new RadioButton( canvas, create.x + create.width + 10, oy, "erase", false, mouseHandler);
			move = new RadioButton( canvas, erase.x + erase.width + 10, oy, "move", false, mouseHandler);
			insert = new RadioButton( canvas, move.x + move.width + 10, oy, "insert", false, mouseHandler);
			create.groupName = erase.groupName = move.groupName = insert.groupName = "tool";
			
			updateButton = new PushButton( canvas, 0, 0, 'update' );
			
			updateButton.x = - updateButton.width / 2;
			updateButton.y = _height *.5 - updateButton.height * 2 - 10;
			updateButton.addEventListener( MouseEvent.MOUSE_DOWN, updateHandler );
			
			canvas.addEventListener( MouseEvent.MOUSE_MOVE, mouseHandler );
			canvas.addEventListener( MouseEvent.MOUSE_DOWN, mouseHandler );
			canvas.addEventListener( MouseEvent.MOUSE_UP, mouseHandler );
			
			points = new Vector.<Point>();
			
			drawGrid( _gridSize );
			addEventListener( Event.ENTER_FRAME, oef );
			
		}
		
		private function oef(e:Event):void 
		{
			redrawTempCanvas();
			var mouse:Point = new Point( canvas.mouseX, canvas.mouseY );
			if ( snap.selected )
			{
				mouse.x = int( ( mouse.x / _gridSize )  ) * _gridSize; 
				mouse.y = int( ( mouse.y / _gridSize )  ) * _gridSize; 
			}
			
			if ( erase.selected )
			{
				highlight( getNearestPoint( mouse ), 0xFF0000 );
			}
			if ( move.selected )
			{
				highlight( getNearestPoint( mouse ), 0x00FF00 );
			}
			if ( insert.selected )
			{
				highlightSegment( mouse );
			}
		}
		
		private function updateHandler(e:MouseEvent):void 
		{
			dispatchEvent( new Event( CHANGE ) );
		}
		
		private function mouseHandler(e:Event):void 
		{
			
			if ( e.target == snap || e.target == create || e.target == erase || e.target == move || e.target == insert || e.target == updateButton )
			{
				return;
			}
			
			var p:Point = new Point( canvas.mouseX, canvas.mouseY ); 
			if ( snap.selected )
			{
				p.x = int( ( p.x / _gridSize ) ) * _gridSize; 
				p.y = int( ( p.y / _gridSize ) ) * _gridSize; 
			}
			
			switch( e.type )
			{
				case MouseEvent.MOUSE_DOWN:
					
					if ( move.selected ) 
					{
						
						_selected = getNearestPoint( p );
						
					}
					else if ( insert.selected )
					{
						
						var index:int = getNearestSegment( p );
						
						if ( snap.selected ) p = project( p, points[ index ], points[ index +1 ] );
						
						points.splice( index + 1, 0, p );
						if( points.length > 1 ) dispatchEvent( new Event( CHANGE ) );
						
					}
					
					break;
					
				case MouseEvent.MOUSE_UP:
					
					if ( erase.selected )
					{
						
						if ( points.length == 0 ) return;
						p = getNearestPoint( p );
						if( points.length > 1 ) points.splice( points.indexOf( p ), 1 );
						
					}
					else if ( create.selected )
					{
						
						points.push( p );
						if( points.length > 1 ) dispatchEvent( new Event( CHANGE ) );
						
					}
					else if (  move.selected && _selected != null )
					{
						
						_selected = null;
						
					}
					
					break;
					
				case MouseEvent.MOUSE_MOVE:
					
					if ( move.selected && _selected != null )
					{
						
						_selected.x = p.x;
						_selected.y = p.y;
						dispatchEvent( new Event( UPDATE ) );
						
					}
					
					break;
			}
			
			
		}
		
		private function highlight(p:Point, color:uint):void 
		{
			
			if ( p == null ) return;
			tempCanvas.graphics.lineStyle( 2, color );
			tempCanvas.graphics.drawCircle( p.x, p.y, 5 );
			
		}
		
		private function highlightSegment( mouse:Point ):void
		{
			
			var index:int = getNearestSegment( mouse );
			var closest:Point = project( mouse, points[ index ], points[ index +1 ] );
			
			tempCanvas.graphics.lineStyle( 2, 0xFF000 );
			tempCanvas.graphics.moveTo( points[ index ].x, points[ index ].y );
			tempCanvas.graphics.lineTo( points[ index + 1 ].x, points[ index + 1 ].y );
			
			tempCanvas.graphics.drawCircle( closest.x, closest.y, 5 );
			
		}
		
		
		
		private function redrawTempCanvas( ):void 
		{
			
			if ( points.length == 0 ) return;
			tempCanvas.graphics.clear();
			tempCanvas.graphics.lineStyle( 2, 0xFFCC00 );
			tempCanvas.graphics.moveTo( points[0].x, points[0].y );
			
			for (var i:int = 0; i < points.length; i++) 
			{
				
				var p:Point = points[ i ];
				
				tempCanvas.graphics.lineTo( p.x, p.y );
				tempCanvas.graphics.drawCircle( p.x, p.y, 3 );
				tempCanvas.graphics.moveTo( p.x, p.y );
				
			}
			
		}
		
		private function getNearestPoint( mouse:Point ):Point
		{
			var dist:Number;
			var minDist:Number = Number.POSITIVE_INFINITY;
			var closest:Point = null;
			for each( var p:Point in points )
			{
				dist = Point.distance( p, mouse );
				if ( dist < minDist )
				{
					minDist = dist;
					closest = p;
				};
			}
			return closest;
		}
		
		private function getNearestSegment(mouse:Point):int 
		{
			var tmp:Vector.<Point> = new Vector.<Point>();
			
			for (var i:int = 0; i < points.length - 1; i++) 
			{
				var cp:Point = points[ i ];
				var np:Point = points[ i + 1 ];
				
				tmp.push( project( mouse, cp, np ) );
				
			}
			
			i = 0;
			var dist:Number;
			var minDist:Number = Number.POSITIVE_INFINITY;
			var closest:Point = null;
			var index:int = 0;
			for each( var tp:Point in tmp )
			{
				dist = Point.distance( tp, mouse );
				if ( dist < minDist )
				{
					minDist = dist;
					closest = tp;
					index = i;
				};
				i++;
			}
			return index;
		}
		
		private function drawGrid( step:uint ):void 
		{
			canvas.graphics.clear();
			
			canvas.graphics.beginFill( 0x333333 );
			canvas.graphics.drawRect( - _width * .5, - _height * .5, _width, _height );
			
			canvas.graphics.lineStyle( 1, 0x444444 );
			for (var i:int = 0; i < _width; i+= step ) 
			{
				canvas.graphics.moveTo( i - _width *.5, -_height * .5 );
				canvas.graphics.lineTo( i - _width *.5, _height );
			}
			
			for (var j:int = 0; j < _height; j+= step ) 
			{
				canvas.graphics.moveTo( -_width * .5, j - _height *.5 );
				canvas.graphics.lineTo( _width, j - _height *.5 );
			}
			
			canvas.graphics.lineStyle( 1, 0x666666 );
			
			canvas.graphics.moveTo( 0, -_height *.5 );
			canvas.graphics.lineTo( 0, _height );
			
			canvas.graphics.moveTo( -_width * .5, 0 );
			canvas.graphics.lineTo( _width, 0 );
			
		}
		
		private function barycenter( p:Point, a:Point, b:Point ):Number
		{
			var len:Number = Point.distance( a, b );
			return (((a.y - p.y) * (a.y - b.y)) - ((a.x - p.x) * (b.x - a.x))) / (len * len);
		}
		
		private function project( p:Point, a:Point, b:Point ):Point
		{
			var r:Number = barycenter( p, a, b );
			r = ( r < 0 ) ? 0 : ( r > 1 ) ? 1 : r;
			return new Point(	a.x + r * (b.x - a.x),
								a.y + r * (b.y - a.y)	);
			
		}
		
		public function get points():Vector.<Point> 
		{
			return _points;
		}
		
		public function set points(value:Vector.<Point>):void 
		{
			_points = value;
		}
		
	}

}