package  geometry._theory 
{
	import away3d.core.math.Quaternion;
	import away3d.primitives.Sphere;
	import flash.geom.Matrix3D;
	import flash.geom.Orientation3D;
	import flash.geom.Vector3D;
	import triga.shapes.Tick;
	import triga.shapes.XYZ;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class I_QuaternionTest extends BaseScene
	{

		private var precision:int = 2;
		private var vectorSphere:Sphere;
		private var quaternionTick:Tick;
		
		public function I_QuaternionTest() 
		{
			setup();
			addChild( GlobalTextField.instance );
			addMesh( 	new XYZ( 100 ), 
						quaternionTick = new Tick( null, 32, RED ),
						vectorSphere = new Sphere( BLUE, 8 ) );
			
			//so, to keep it simple we have this object located at 100, 0, 0
			var pos:Vector3D = new Vector3D( 100,0,0 );
			addMesh( new Tick( pos, 25, WHITE ) );
			
			var radians:Number = Math.PI / 4; // = 45°, but we're not kids anymore :)
			
			// what is a Quaternion?
			//basically a point in 3D described by 3 components X / Y / Z + a W value
			//the XYZ describe an axis and the W is an angle of rotation around this axis
			
			var q:Quaternion = new Quaternion( 0, 0, 0, 1 );
			
			//ho ho ho! 
			
			//we saw that a vector also has a W property that had something to do with quaternion ( see A_VectorProperties.as )
			var v:Vector3D = new Vector3D( 0, 0, 0, 1 );
			
			// so basically, a Quaternion is a point in 3D ( considered as an axis ), that stores a value ( usually W )
			
			//now in most implementations, there should be something like:
			q.fromEulerAngles( radians, radians, radians );
			
			//hmmmmm but we can also assign a rotation to a Matrix right ?
			var m:Matrix3D = new Matrix3D();
			m.recompose( Vector.<Vector3D>( [ new Vector3D(), new Vector3D(radians, radians, radians), new Vector3D(1, 1, 1) ] ) );
			
			//and the Matrix3D also has a "decompose" method to retrieve our P / R / S components as Vector3D.
			//overmore we can format the output result as a Quaternion:
			v = m.decompose( Orientation3D.QUATERNION )[ 1 ];
			
			//now tracing the components of bothe the quaternion and the vector3D will give us:
			_trace( 'quaternion values ->', q.x.toFixed( precision ), q.y.toFixed( precision ), q.y.toFixed( precision ), q.w.toFixed( precision ) );
			_trace( 'vector3D values ->', v.x.toFixed( precision ), v.y.toFixed( precision ), v.y.toFixed( precision ), v.w.toFixed( precision ) );
			
			//the same result
			//...ah... 
			//so?
			//well... let's use it...
			
			quaternionTick.position = q.rotatePoint( pos );
			
			vectorSphere.position = m.transformVector( pos );
			
			//hmm ok so it's kind of the same right?
			//a Quaternion is meant to be somewhat simpler to use
			//as we saw, combining the rotations with EULER angles will give different results depending on the order.
			// x*y*z != x*z*y != z*y*x ...
			//quaternions have a way of combining the angles together so that we can perform the rotation "at once" around an arbitrary axis.
			//so, most of all a Quaternion will prevent the Gimbal Lock.
			
		}
		
	}

}