package geometry.basics 
{
	import away3d.core.base.Geometry;
	import away3d.core.base.SubGeometry;
	import away3d.entities.Mesh;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class B_CustomGeometry extends BaseScene 
	{
		/**
		 * 
		 * cf. lib/doc/CSE167_05.pdf p 37 à 44
		 * 
		 */
		
		public function B_CustomGeometry() 
		{
			setup();
			
			//we create an empty Mesh object
			var mesh:Mesh = new Mesh();
			
			//this object can hold multiple subgeometries
			var geometry:Geometry = mesh.geometry;
			
			//this is an object that holds the vertices and the indices ( + some more stuff )
			var subGeometry:SubGeometry = new SubGeometry();
			
			//vertex buffer
			//the vertices ( the locations of the points in 3D ) are a Vector of Numbers that 
			//describe each vertex' X Y Z coordinates as triplets of numbers
			
			var vertices:Vector.<Number> = new Vector.<Number>();
			
			vertices.push( -1,  1,	0 );//vertex 0 -> x : -1 / y : 0 / z :  1
			vertices.push(  1,  1,	0 );//vertex 1 -> x :  1 / y : 0 / z :  1
			vertices.push(  1, -1,	0 );//vertex 2 -> x :  1 / y : 0 / z : -1
			vertices.push( -1, -1,	0 );//vertex 3 -> x : -1 / y : 0 / z : -1
			
			//to update the vertex buffers we call updateVertexData on the subgeometry 
			subGeometry.updateVertexData( vertices );
			
			
			//index buffer
			//the indices describe how to associate the vertices into triangles.
			//it's a Vector of uints referring to the position of the triangles' vertices in the vertex buffer
			
			var indices:Vector.<uint> = new Vector.<uint>();
			
			indices.push( 0, 1, 2 );// triangle 0 = vertex 0 - vertex 1 - vertex 2
			indices.push( 0, 2, 3 );// triangle 1 = vertex 0 - vertex 2 - vertex 3
			
			//to update the index buffers we call updateIndexData on the subgeometry 
			subGeometry.updateIndexData( indices );
			
			//this adds the current subGeometry to the Mesh's geometry
			geometry.addSubGeometry( subGeometry );
			
			
			//adding to scene
			WHITE.lights = lights;
			mesh.material = WHITE;
			mesh.scale( 100 );
			addMesh( mesh );
			
		}
		
	}

}