package  geometry.basics 
{
	import away3d.core.base.SubGeometry;
	import away3d.entities.Mesh;
	import flash.events.KeyboardEvent;
	import flash.geom.Vector3D;
	import triga.shapes.CrossHair;
	import triga.shapes.Tick;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class C_UnprojectPlane extends BaseScene 
	{
		
		private var tick:Tick;
		
		public function C_UnprojectPlane() 
		{
			setup();
			
			camera.lens.near = 100;
			
			addMesh( tick = new Tick(new Vector3D, 25, WHITE ) );
			
			addMesh( new CrossHair( 100, 100, 100 ) );
			
			YELLOW.alpha = .5;
			
			addChild( GlobalTextField.instance );
			_trace( 'give focus then\npress any key to unproject the view plane and build a quadrilateral' );
		}
		
		override protected function onKeyDown(e:KeyboardEvent):void 
		{
			
			unprojectMouse();
			
			createViewPlane();
			
		}
		
		private function unprojectMouse():void 
		{
			
			var mouse:Vector3D = view.unproject( view.mouseX, view.mouseY );
			
			var cam:Vector3D = view.camera.position;
			
			var dir:Vector3D = mouse.subtract( cam );
			
			dir.normalize();
			
			//we use the camera's lens' near plane's distance to position the mouse's cursor 
			dir.scaleBy( view.camera.lens.near );
			
			tick.position = camera.position.add( dir );
			
		}
		
		private function createViewPlane():void 
		{
			//unprojecting the screen coordinates
			var tl:Vector3D = view.unproject( 0, 0 );
			var tr:Vector3D = view.unproject( view.width, 0 );
			var br:Vector3D = view.unproject( view.width, view.height );
			var bl:Vector3D = view.unproject( 0, view.height );
			
			//createViewPlane vertices
			var vertices:Vector.<Number> = Vector.<Number>( [
															tl.x, tl.y, tl.z,
															tr.x, tr.y, tr.z,
															br.x, br.y, br.z,
															bl.x, bl.y, bl.z
															] );
			
			//createViewPlane indices
			var indices:Vector.<uint> = Vector.<uint>( [
														0, 1, 2,
														0, 2, 3
														]);
			
			//make it double sided
			indices = indices.concat( indices.concat().reverse() );
			
			//create a mesh
			var mesh:Mesh = new Mesh( YELLOW, null );
			
			//create a subgeometry to hold the vertices and indices
			var subGeometry:SubGeometry = new SubGeometry();
			
			//updates vertices
			subGeometry.updateVertexData( vertices );
			
			//updates indices
			subGeometry.updateIndexData( indices );
			
			//update mesh with new subGeometry
			mesh.geometry.addSubGeometry( subGeometry );
			
			addMesh( mesh );
			
			
		}
		
	}

}