package  geometry.curves
{
	import away3d.entities.Sprite3D;
	import away3d.materials.BitmapMaterial;
	import com.bit101.components.CheckBox;
	import com.bit101.components.HUISlider;
	import com.bit101.components.PushButton;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.geom.Vector3D;
	import triga.render.MaterialHelper;
	import triga.render.ViewHelper;
	import triga.spline.curves.Cardinal;
	import triga.spline.curves.CatmullRom;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.Spline;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class B_SplineTypes extends BaseScene 
	{
		[Embed(source = "../../../lib/res/textures/crosshair.png")]
		private var textureSrc:Class;
		
		private var crosshair:BitmapData = new textureSrc().bitmapData;
		
		private var spline:Spline;
		private var path:Vector.<Vector3D>;
		
		private var cubic:Cubic;
		private var catmull:CatmullRom;
		private var cardinal:Cardinal;
		
		private var splineCubic:Spline;
		private var splineCatmull:Spline;
		private var splineCardinal:Spline;
		
		private var splines:Vector.<Spline> = new Vector.<Spline>();
		
		/***********************************/
		
		private var cb_cub:CheckBox;
		private var cb_cat:CheckBox;
		private var cb_car:CheckBox;
		
		private var pr_cub:HUISlider;
		private var pr_cat:HUISlider;
		private var pr_car:HUISlider;
		private var te_car:HUISlider;
		
		private var cb_loop:CheckBox;
		
		public function B_SplineTypes() 
		{
			lightColors = [ 0, 0xFFFFFF, 0, 0xFFFFFF, 0 ];//0xFFFFFF, 0xFFFFFF, 0xFFFFFF ];
			setup();
			ViewHelper.grainyBackground( view, 1024, true, 216, 255 );
			
			var size:Number = 125;
			path = Vector.<Vector3D>(	[
											new Vector3D( -size, -size, -size ),
											new Vector3D(  size, -size, -size ),
											new Vector3D(  size,  size, -size ),
											new Vector3D( -size,  size, -size ),
											new Vector3D( -size, -size,  size ),
											new Vector3D(  size, -size,  size ),
											new Vector3D(  size,  size,  size ),
											new Vector3D( -size,  size,  size )
										]	);
			
			var crossMaterial:BitmapMaterial = new BitmapMaterial( crosshair );
			crossMaterial.alphaBlending = true;
			ViewHelper.vignette( view, 1024, false, 0xFFFFFF, 1 );
			for each( var v:Vector3D in path )
			{
				var sp3d:Sprite3D = new Sprite3D( crossMaterial, 64,64);
				addMesh( sp3d );
				sp3d.position = v;
			}
			
			/***********************
			
			create a cubic path
			
			***********************/
			
			cubic = new Cubic( .1 );
			
			var mat_cub:BitmapMaterial = MaterialHelper.horizontalGradient(64, [0xFFCC0033, 0xFFFF0099] );
			splineCubic = new Spline( cubic.compute( path ), 3, 5, false, mat_cub );
			addMesh( splineCubic.mesh );
			
			
			/***********************
			
			create a catmull-rom path
			
			***********************/
			
			catmull = new CatmullRom( .1 );
			
			var mat_cat:BitmapMaterial = MaterialHelper.horizontalGradient(64, [0xFF33CC00, 0xFF99FF00] );
			splineCatmull = new Spline( catmull.compute( path ), 3, 5, false, mat_cat );
			addMesh( splineCatmull.mesh );
			
			
			/***********************
			
			create a cardinal path
			
			***********************/
			
			cardinal = new Cardinal( .1, 0 );
			
			var mat_car:BitmapMaterial = MaterialHelper.horizontalGradient(64, [0xFF0033FF, 0xFF0099FF] );
			splineCardinal = new Spline( cardinal.compute( path ), 3, 5, false, mat_car );
			addMesh( splineCardinal.mesh );
			
			splines.push( splineCubic, splineCatmull, splineCardinal );
			
			
			/***********************
			
				interface
			
			***********************/
			
			var ox:int = 10;
			var oy:int = -10;
			var random:PushButton = new PushButton( this, ox, oy += 20, 'randomize', randomize );
			
			oy += 20;
			cb_cub = new CheckBox( this, ox, oy += 20, 'cubic spline', showHide );
			pr_cub = new HUISlider( this, ox + 100, oy, 'precision', precision );
			
			cb_cat = new CheckBox( this, ox, oy += 20, 'catmull spline', showHide );
			pr_cat = new HUISlider( this, ox + 100, oy, 'precision', precision );
			
			cb_car = new CheckBox( this, ox, oy += 20, 'cardinal spline', showHide );
			pr_car = new HUISlider( this, ox + 100, oy, 'precision', precision );
			te_car = new HUISlider( this, ox + 100, oy+20, 'tension', precision );
			
			pr_cub.value = cubic.precision * 100;
			pr_cat.value = catmull.precision * 100;
			pr_car.value = cardinal.precision * 100;
			
			te_car.minimum = -5;
			te_car.maximum = 5;
			
			cb_cub.selected = cb_cat.selected = cb_car.selected = true;
			
			oy += 20;
			cb_loop = new CheckBox( this, ox, oy, 'loop', onLoop );
			
		}
		
		
		
		private function randomize( e:Event = null ):void 
		{
			
			path.sort( random );
			computeAndUpdate();
			
		}
		
		private function random( a:Vector3D, b:Vector3D ):Number
		{
			
			return Math.random() > .5 ? -1 : 1;
			
		}
		
		private function showHide( e:Event = null ):void 
		{
			
			splineCubic.mesh.visible = cb_cub.selected;
			splineCatmull.mesh.visible = cb_cat.selected;
			splineCardinal.mesh.visible = cb_car.selected;
			
		}
		
		private function precision( e:Event = null ):void 
		{
			
			cubic.precision 	= pr_cub.value / 100;
			catmull.precision 	= pr_cat.value / 100;
			cardinal.precision 	= pr_car.value / 100;
			cardinal.tension 	= te_car.value;
			
			if ( e.target == pr_cub )
			{
				splineCubic.path = cubic.compute( path );
			}
			if ( e.target == pr_cat )
			{
				splineCatmull.path = catmull.compute( path, splineCatmull.path );
			}
			if ( e.target == pr_car || e.target == te_car )
			{
				splineCardinal.path = cardinal.compute( path, splineCardinal.path );
			}
			
		}
		
		private function onLoop( e:Event = null ):void 
		{
			
			cubic.loop = catmull.loop = cardinal.loop = cb_loop.selected;
			
			splineCubic.path = cubic.compute( path );
			splineCatmull.path = catmull.compute( path, splineCatmull.path );
			splineCardinal.path = cardinal.compute( path, splineCardinal.path );
			
		}
		
		private function computeAndUpdate():void
		{
			cubic.compute( path, splineCubic.path );
			splineCubic.update();
			
			catmull.compute( path, splineCatmull.path );
			splineCatmull.update();
			
			cardinal.compute( path, splineCardinal.path );
			splineCardinal.update();
		}
	}

}