package geometry.extrusion 
{
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import triga.spline.shapes.Section;
	import triga.spline.shapes.sections.Frame;
	import triga.spline.shapes.sections.Star;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class A_SectionTest extends BaseScene 
	{
		
		public function A_SectionTest() 
		{
			setup();
			
			//a Section object is a 2D polygon that renders in 3D
			//it has to be ordered and simple ; as the set of points will be triangulated, 
			//the points of the polygon must be sorted and the polygon must not self-intersect
			
			//so first we need to describe a 2D shape here's a triangle
			var points:Vector.<Point> = Vector.<Point>([
														new Point( 50,  40 ),
														new Point( -50, 40 ),
														new Point( 0, -40 )
														]);
			
			//and then we can create a 2D object in 3D with this shape
			var section:Section = new Section( points, RED );
			addMesh( section );
			
			/*
			//there is a static helper method to create Regular Polygons
			removeMesh( section );
			section = Section.createRegularSection( 50, 5, RED );
			//will result in a red pentagon of radius 50
			addMesh( section );
			//*/
			
			
			/*
			//there's also a method to inject X/Y doublets and create a shape from it.
			//there are several JSFL vectorization utilities around ( the one I used is in /lib/res/fla/jsfl/vectorize.jsfl )
			
			//it can also be performed 'live' with vectorisation algorithms for more info see:
			// http://www.sakri.net/blog/2009/04/13/extract-shape-outline-points-from-bitmapdata/
			// http://www.sakri.net/blog/2009/04/28/extracting-analyzing-and-optimizing-a-vector-shape-from-a-bitmapdata-shape/
			
			removeMesh( section );
			section = Section.createFromValues(	Vector.<Number>([6.35, -42, 11.85, -34.75, 12.85, -27.75, 43.35, -27.75, 43.35, -52, 124.1, -52, 124.1, -37.25, 99.1, -25.75, 99.1, 41.75, 62.6, 41.75, 61.1, 49.75, 55.85, 57.25, 48.35, 63.25, 40.35, 66.5, 33.1, 67, 25.85, 65.75, 19.35, 63, 12.1, 57.25, 7.35, 49.5, 5.85, 42, -21.65, 42, -17.9, 48, -17.4, 53.5, -19.9, 59.25, -24.4, 65, -31.15, 67.25, -40.15, 65.75, -45.9, 62.25, -49.4, 56.75, -49.9, 50.75, -47.65, 43.75, -45.9, 41.75, -60.4, 41.75, -56.9, 49, -57.65, 54, -59.9, 60.25, -64.4, 65, -70.9, 67.25, -77.9, 66.5, -85.65, 61.75, -88.65, 57, -89.65, 50.5, -87.65, 44.5, -85.65, 41.5, -95.65, 41.5, -95.65, 58.25, -131.65, 58.5, -105.9, 36.5, -114.65, 26, -117.9, 12.75, -123.4, 12.75, -123.4, -4.5, -118.15, -4.5, -114.4, -17.25, -106.65, -27.5, -79.4, -27.5, -92.65, -71.25, -80.65, -87, -80.4, -90.25, -55.9, -90.25, -55.9, -87.5, -44.65, -71, -58.4, -27.25, -24.4, -27.25, -22.4, -35, -16.15, -42.25, -7.9, -45.75, -0.4, -45.75]), WHITE );
			addMesh( section );
			//*/
			
			
			/*
			section.x -= 200;
			
			//there are some ready made Sections to help us:
			//here's a Star object
			
			var star:Section = new Star( 5, 31, 50, BLUE );
			addMesh( star );
			
			//*/
			
			/*
			star.x += 150;
			
			//here's a Frame object that takes 2 rectangles and creates a frame
			
			var inner:Rectangle = new Rectangle( -20, -40, 40, 80 );
			var outer:Rectangle = new Rectangle( -30, -50, 60, 100 );
			
			var frame:Frame = new Frame( inner, outer, YELLOW );
			addMesh( frame );
			
			//*/
			
			
		}
		
	}

}