package geometry.extrusion 
{
	import away3d.core.base.SubGeometry;
	import away3d.core.partition.MeshNode;
	import away3d.entities.Mesh;
	import flash.geom.Vector3D;
	import triga.spline.shapes.Section;
	import triga.utils.MeshUtils;
	
	
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class B_LinearExtrusion extends BaseScene 
	{
		
		public function B_LinearExtrusion() 
		{
			
			setup();
			
			//we create a new section
			var section:Section = Section.createFromValues(	Vector.<Number>([6.35, -42, 11.85, -34.75, 12.85, -27.75, 43.35, -27.75, 43.35, -52, 124.1, -52, 124.1, -37.25, 99.1, -25.75, 99.1, 41.75, 62.6, 41.75, 61.1, 49.75, 55.85, 57.25, 48.35, 63.25, 40.35, 66.5, 33.1, 67, 25.85, 65.75, 19.35, 63, 12.1, 57.25, 7.35, 49.5, 5.85, 42, -21.65, 42, -17.9, 48, -17.4, 53.5, -19.9, 59.25, -24.4, 65, -31.15, 67.25, -40.15, 65.75, -45.9, 62.25, -49.4, 56.75, -49.9, 50.75, -47.65, 43.75, -45.9, 41.75, -60.4, 41.75, -56.9, 49, -57.65, 54, -59.9, 60.25, -64.4, 65, -70.9, 67.25, -77.9, 66.5, -85.65, 61.75, -88.65, 57, -89.65, 50.5, -87.65, 44.5, -85.65, 41.5, -95.65, 41.5, -95.65, 58.25, -131.65, 58.5, -105.9, 36.5, -114.65, 26, -117.9, 12.75, -123.4, 12.75, -123.4, -4.5, -118.15, -4.5, -114.4, -17.25, -106.65, -27.5, -79.4, -27.5, -92.65, -71.25, -80.65, -87, -80.4, -90.25, -55.9, -90.25, -55.9, -87.5, -44.65, -71, -58.4, -27.25, -24.4, -27.25, -22.4, -35, -16.15, -42.25, -7.9, -45.75, -0.4, -45.75]), WHITE );
			
			//we need an access to the subgeometry data
			var subGeometry:SubGeometry = section.geometry.subGeometries[ 0 ];
			
			//first we need to create the new vertices
			var vertices:Vector.<Number> = subGeometry.vertexData.concat();
			vertices.fixed = false;
			
			
			//sides is the number of vertices of the polygon = the vertex buffer length / 3
			var sides:uint = vertices.length / 3;
			
			var len:uint = vertices.length;
			for ( var i:int = 0; i < len; i += 3 )
			{
				
				//rertrieve a vector from the vertex buffer
				
				var v:Vector3D = new Vector3D(  vertices[ i ],
												vertices[ i + 1 ],
												vertices[ i + 2 ] );
				
				//adds a new vertex to the vertex buffer
				
				vertices.push( v.x, v.y, v.z + 50 );
				
			}
			
			//collect the indices
			var indices:Vector.<uint> = subGeometry.indexData.concat();
			indices.fixed = false;
			
			//we know that the section is doublesided 
			//so we only need to offset half of the existing indices by the sides count
			
			len = indices.length;
			for ( i = len / 2; i < len; i += 3 )
			{
				
				indices[ i ] += sides;
				indices[ i + 1 ] += sides;
				indices[ i + 2 ] += sides;
				
			}
			
			//and this is "turning around" the polygon and creates 2 faces 
			//between the first vertices and the newly created ones
			//like a 'bridge' between the 2 sides
			
			var i0:uint, i1:uint, i2:uint, i3:uint;
			for ( i = 0; i < sides; i++ )
			{
				
				i0 = i;
				i1 = ( i + 1 ) % sides;
				i2 = sides + i;
				i3 = sides + ( i + 1 ) % sides;
				
				indices.push( i0, i1, i2 );
				indices.push( i2, i1, i3 );
				
			}
			
			//then we update the mesh with the new vertices and indices
			
			subGeometry.updateIndexData( indices );
			subGeometry.updateVertexData( vertices );
			
			section.material = WHITE;
			WHITE.alpha = .5;
			addMesh( section );
			
		}
	}

}