package geometry.loft 
{
	import flash.geom.Vector3D;
	import triga.shapes.Axis;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.Section;
	import triga.spline.shapes.Spline;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class B_SmootherNaiveLoft extends BaseScene 
	{
		
		public function B_SmootherNaiveLoft() 
		{	
			setup();
			BLUE.alpha = .5;
			
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
			
			path.push( 	
						new Vector3D( -100, -100, 0 ),
						new Vector3D( 0, -100, 0 ),
						new Vector3D( 100, 0, 100 ),
						new Vector3D( 100, 0, 200 ),
						new Vector3D( -100,100,-100 ),
						new Vector3D( 0, 200, 100 ),
						new Vector3D( -50, 250, 50 )	
						);
			
			//this will smooth the path
			path = new Cubic( .1, false ).compute( path );
			
			computeNaiveLoft( path );
			
		}
		
		private function computeNaiveLoft( path:Vector.<Vector3D> ):void 
		{
			
			var nextPosition:Vector3D;
			
			var currentSection:Section;
			var lastSection:Section;
			for (var i:int = 0; i < path.length; i++) 
			{
				
				//finds the next location on the path
				nextPosition = ( i < path.length - 1 ) ? path[ i + 1 ] : path[ i - 1 ];
				
				//if the next position is almost equal to the current position, nothing to process
				if ( nextPosition.nearEquals( path[ i ], 1 ) ) continue;
				
				//create a new section object
				currentSection = Section.createRegularSection( 30, 3, BLUE );
				
				//positions it on the current location
				currentSection.position = path[ i ];
				
				//make the current section point towards the next location
				currentSection.lookAt( nextPosition );
				addMesh( currentSection );
				
				//draws a segment between the current and the next location
				var axis:Axis = new Axis( path[ i ], path[ i + 1 ], 2, WHITE );
				addMesh( axis );
				
				
				//creates a set of lines between the vectors of the current and the previous shape
				if ( lastSection != null )
				{
					var colors:Array = [ GREEN, YELLOW, RED ];
					
					var last_vectors:Vector.<Vector3D> = lastSection.vectors3D;
					var new_vectors:Vector.<Vector3D> = currentSection.vectors3D;
					
					for (var j:int = 0; j < last_vectors.length; j++) 
					{
						var _last:Vector3D = last_vectors[ j ];
						var _new:Vector3D = new_vectors[ j ];
						
						var sideAxis:Axis = new Axis( _last, _new, 1, colors[ j % colors.length ] );
						addMesh( sideAxis );
						
					}
					
				}
				lastSection = currentSection;
				
			}
		}
	}

}