package geometry.loft 
{
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import triga.shapes.Axis;
	import triga.shapes.XYZ;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.path.Helix;
	import triga.spline.shapes.Section;
	import triga.spline.shapes.Spline;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class C_FrenetFrame extends BaseScene 
	{
		
		public function C_FrenetFrame() 
		{	
			setup();
			WHITE.alpha = .25;
			
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
						
			//if the path is regular: ok under certain conditions
			//a regular upwards Helix : ok
			path = new Helix( 3, 250, 100, 100, 12 ).path;
			
			//a horizontal Helix : no
			//path = new Helix( 3, 250, 100, 100, 12, Vector3D.X_AXIS ).path;
			
			//an arbytrary shape : ouch!
			///*
			path = new Vector.<Vector3D>()
			path.push( 	
						new Vector3D( -100, -100, 0 ),
						new Vector3D( 0, -100, 0 ),
						new Vector3D( 100, 0, 100 ),
						new Vector3D( 100, 0, 200 ),
						new Vector3D( -100,100,-100 ),
						new Vector3D( 0, 200, 100 ),
						new Vector3D( -50, 250, 50 )	
						);
			path = new Cubic( .1, false ).compute( path );
			//*/
			computeFremetFrame( path );
			
		}
		
		private function computeFremetFrame( path:Vector.<Vector3D> ):void 
		{
			
			//we'll have to compute the tangent of 2 segments: the previous-current and the current-next
			var prev:Vector3D, vector:Vector3D, next:Vector3D;
			
			var m:Matrix3D = new Matrix3D();
			var currentSection:Section;
			var lastSection:Section;
			for (var i:int = 0; i < path.length; i++) 
			{
				
				//finds the previous, current and next locations on the path
				prev = ( i > 0 ) ? path[ i - 1 ] : path[ 0 ];
				vector = path[ i ];
				next = ( i < path.length - 1 ) ? path[ i + 1 ] : path[ i - 1 ];
				
				//2 locations are coincident, skip
				if ( prev.nearEquals( vector, 1 ) ) continue;
				if ( next.nearEquals( vector, 1 ) ) continue;
				
				//T is the unit vector tangent to the curve, pointing in the direction of motion.
				var T:Vector3D = prev.subtract( vector );
				T.normalize();
				
				//N is the normalized derivative of T with respect to the arclength parameter of the curve.
				var N:Vector3D = T.crossProduct( vector.subtract( next ) );
				N.normalize();
				
				//B is the cross product of T and N.
				var B:Vector3D = T.crossProduct( N );
				
				//create a new section object
				currentSection = Section.createRegularSection( 15, 3, WHITE );
				m.identity();
				m.pointAt( T, Vector3D.Z_AXIS, Vector3D.Y_AXIS );
				currentSection.transform = m;
				currentSection.position = vector;
				trace( 'hop' );
				var size:Number = Vector3D.distance( vector, next );
				
				T.scaleBy( size );
				N.scaleBy( size );
				B.scaleBy( size );
				
				T = T.add( vector );
				N = N.add( vector );
				B = B.add( vector );
				
				addMesh( 	currentSection,
							new Axis( vector, T, 1, RED ),
							new Axis( vector, N, 1, GREEN ),
							new Axis( vector, B, 1, BLUE ) 	);
				
				//creates a set of lines between the vectors of the current and the previous shape
				if ( lastSection != null )
				{
					var colors:Array = [ GREEN, YELLOW, RED ];
					
					var last_vectors:Vector.<Vector3D> = lastSection.vectors3D;
					var new_vectors:Vector.<Vector3D> = currentSection.vectors3D;
					
					for (var j:int = 0; j < last_vectors.length; j++) 
					{
						
						var _last:Vector3D = last_vectors[ j ];
						var _new:Vector3D = new_vectors[ j ];
						addMesh( new Axis( _last, _new, 1, colors[ j % colors.length ] ) );
						
					}
				}
				lastSection = currentSection;
				
			}
		}
	}

}