package geometry.loft 
{
	import away3d.primitives.Cube;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import triga.shapes.Axis;
	import triga.shapes.XYZ;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.Loft;
	import triga.spline.shapes.path.Knot;
	import triga.spline.shapes.Section;
	import triga.spline.shapes.Spline;
	import triga.utils.VectorUtils;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class D_FixedUpVector extends BaseScene 
	{
		
		public function D_FixedUpVector() 
		{	
			setup();
			WHITE.alpha = .5;
			
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
			
			path.push( 	
						new Vector3D( -100, -100, 0 ),
						new Vector3D( 0, -100, 0 ),
						new Vector3D( 100, 0, 100 ),
						new Vector3D( 100, 0, 200 ),
						new Vector3D( -100,100,-100 ),
						new Vector3D( 0, 200, 100 ),
						new Vector3D( -50, 250, 50 )	
						);
			
			//this will smooth the path
			path = new Cubic( .1, false ).compute( path );
			computeNaiveLoft( path );
			
		}
		
		private function computeNaiveLoft( path:Vector.<Vector3D> ):void 
		{
			
			//we'll have to compute the tangent of 2 segments: the previous-current and the current-next
			var vector:Vector3D;
			var T:Vector3D, N:Vector3D, B:Vector3D;
			
			//let's precompute the tangents
			var tangents:Vector.<Vector3D> = new Vector.<Vector3D>();
			tangents.push( path[ 1 ].subtract( path[ 0 ] ));
			for (var i:int = 1; i < path.length - 1; i++) 
			{
				tangents.push( path[ i + 1 ].subtract( path[ i - 1 ] ) );
			}
			tangents.push( path[ path.length - 1 ].subtract( path[ path.length - 2 ] ));
			
			//let's precompute the normals
			var normals:Vector.<Vector3D> = new Vector.<Vector3D>();
			for ( i = 0; i < path.length; i++) 
			{
				T = tangents[ i ];
				N = tangents[ i > 1 ? (i - 1) : 0 ].subtract( T );
				normals.push( N );
			}
			
			//and let's precompute the BInormals too !
			var binormals:Vector.<Vector3D> = new Vector.<Vector3D>();
			for ( i = 0; i < path.length; i++) 
			{
				T = tangents[ i ];
				N = normals[ i ];
				B = T.crossProduct( N );
				binormals.push( B );
			}
			
			
			
			var currentSection:Section;
			var lastSection:Section;
			for ( i = 0; i < path.length; i++) 
			{
				vector = path[ i ];
				
				var up:Vector3D = Vector3D.Y_AXIS;
				T = tangents[ i ];
				N = T.crossProduct( up );
				N.normalize();
				B = N.crossProduct( T );
				
				currentSection = Section.createRegularSection( 15, 3, WHITE );
				var m:Matrix3D = new Matrix3D();
				m.pointAt( T, Vector3D.Z_AXIS, Vector3D.Y_AXIS );
				currentSection.transform = m;
				currentSection.position = vector;
				addMesh( currentSection );
				
				var size:Number = Vector3D.distance( path[ i ], path[ i - 1 < 0 ? 0 :( i-1 ) ]);
				
				T.normalize();
				N.normalize();
				B.normalize();
				
				T.scaleBy( size );
				N.scaleBy( size );
				B.scaleBy( size );
				
				T = T.add( vector );
				N = N.add( vector );
				B = B.add( vector );
				
				addMesh( 	new Axis( vector, T, 1, RED ),
							new Axis( vector, N, 1, GREEN ),
							new Axis( vector, B, 1, BLUE ) 	);
				
							
							
				//creates a set of lines between the vectors of the current and the previous shape
				if ( lastSection != null )
				{
					var colors:Array = [ GREEN, YELLOW, RED ];
					
					var last_vectors:Vector.<Vector3D> = lastSection.vectors3D;
					var new_vectors:Vector.<Vector3D> = currentSection.vectors3D;
					
					for (var j:int = 0; j < last_vectors.length; j++) 
					{
						var _last:Vector3D = last_vectors[ j ];
						var _new:Vector3D = new_vectors[ j ];
						
						var sideAxis:Axis = new Axis( _last, _new, 1, colors[ j % colors.length ] );
						addMesh( sideAxis );
						
					}
					
				}
				lastSection = currentSection;
				
			}
		}
	}

}