package geometry.loft 
{
	import away3d.core.math.Quaternion;
	import away3d.primitives.Cube;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	import triga.shapes.Axis;
	import triga.shapes.XYZ;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.Loft;
	import triga.spline.shapes.path.Helix;
	import triga.spline.shapes.path.Knot;
	import triga.spline.shapes.Section;
	import triga.spline.shapes.Spline;
	import triga.utils.VectorUtils;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class E_ParallelTransportFrame extends BaseScene 
	{
		
		public function E_ParallelTransportFrame() 
		{	
			setup();
			
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
			
			path.push( 	
						new Vector3D( -100, -100, 0 ),
						new Vector3D( 0, -100, 0 ),
						new Vector3D( 100, 0, 100 ),
						new Vector3D( 100, 0, 200 ),
						new Vector3D( -100,100,-100 ),
						new Vector3D( 0, 200, 100 ),
						new Vector3D( -50, 250, 50 )	
						);
			
			//this will smooth the path
			path = new Cubic( .1, true ).compute( path );
			
			computeParallelTransport( path );
			
		}
		
		private function computeParallelTransport( path:Vector.<Vector3D> ):void 
		{
			
			var T1:Vector3D, T2:Vector3D, A:Vector3D;
			var angle:Number;
			
			
			var tangents:Vector.<Vector3D> = new Vector.<Vector3D>();
			
			tangents.push( path[ 1 ].subtract( path[ 0 ] ));
			for (var i:int = 1; i < path.length; i++) 
			{
				tangents.push( A = path[ i ].subtract( path[ i - 1 ] ) );
				A.normalize();
			}
			
			var currentSection:Section = Section.createRegularSection( 5, 3, WHITE );
			getFirstFrame( path, currentSection );
			addMesh( currentSection );
			
			var m:Matrix3D = currentSection.transform.clone();
			
			var lastSection:Section = Section.createRegularSection( 5, 3, WHITE );
			lastSection.transform = m;
			
			for ( i = 1; i < path.length; i++) 
			{
				
				//previous tangent
				T1 = tangents[ i - 1  ];
				//next tangent
				T2 = tangents[ i ];
				//axis of rotation
				A = T1.crossProduct( T2 );
				
				// for static pathes only: the direction is the same, we can skip the computation
				if ( A.x == 0 && A.y == 0 && A.z == 0 ) continue;
				
				//amount of rotation around the A axis
				angle = Math.acos( T1.dotProduct( T2 ) ) * 180 / Math.PI ;
				
				m.appendRotation( angle, A );
				
				currentSection.transform = m;
				currentSection.position = path[ i ];
				
				///*
				//test that eliminates some error cases
				//retrieve local Z axis
				m.copyColumnTo( 2, A );
				if ( 	A.nearEquals( Vector3D.X_AXIS, .1 ) 
				||		A.nearEquals( Vector3D.Y_AXIS, .1 ) 
				||		A.nearEquals( Vector3D.Z_AXIS, .1 ) )	continue
				//*/
				
				//creates a set of lines between the vectors of the current and the previous shape
				if ( lastSection != null )
				{
					var colors:Array = [ GREEN, YELLOW, RED ];
					
					var last_vectors:Vector.<Vector3D> = lastSection.vectors3D;
					var new_vectors:Vector.<Vector3D> = currentSection.vectors3D;
					
					for (var j:int = 0; j < last_vectors.length; j++) 
					{
						var _last:Vector3D = last_vectors[ j ];
						var _new:Vector3D = new_vectors[ j ];
						
						var sideAxis:Axis = new Axis( _last, _new, 1, colors[ j % colors.length ] );
						addMesh( sideAxis );
						
					}
				}
				lastSection.transform = currentSection.transform.clone();
				
			}
		}
		
		private function getFirstFrame( path:Vector.<Vector3D>, _section:Section, upAxis:Vector3D = null ):void 
		{	
			//find the orientation of the section at this location
			
			var v:Vector3D = path[ 0 ];
			var n:Vector3D = path[ 1 ];
			var dir:Vector3D = n.subtract( v );
			dir.normalize();
			var angle:Number = Math.acos( ( upAxis||Vector3D.Z_AXIS ).dotProduct( dir ) );
			var axis:Vector3D = ( upAxis||Vector3D.Z_AXIS ).crossProduct(dir);
			axis.normalize();
			_section.transform.identity();
			_section.rotate( axis, angle * 180 / Math.PI );
			_section.position = path[ 0 ];
		}
		
		
	}

}