package geometry.loft 
{
	import away3d.core.base.Object3D;
	import away3d.entities.Mesh;
	import flash.geom.Vector3D;
	import triga.spline.shapes.path.Helix;
	import triga.spline.shapes.path.Knot;
	import triga.spline.shapes.PTFLoft;
	import triga.spline.shapes.Section;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class F_Primitives extends BaseScene 
	{
		
		public function F_Primitives() 
		{
			setup();
			RED.alpha = BLUE.alpha = GREEN.alpha = YELLOW.alpha = .5;
			
			var shapeList:Vector.<Object3D> = new Vector.<Object3D>();
			
			var objectSize:Number = 100;
			
			//a vertical line
			var line:Vector.<Vector3D> = Vector.<Vector3D>( [ new Vector3D( 0,-objectSize*.5,0 ), new Vector3D( 0,objectSize*.5,0 ) ] );
			
			//a disc
			var disc:Helix = new Helix( 1, 0, objectSize * .5, objectSize * .5, 32 );
			
			//an helix
			var helix:Helix = new Helix( 3, objectSize, objectSize * .5, objectSize * .5, 32 );
			
			//an knot
			var knot_path:Knot = new Knot( 2, 3, objectSize * .25, 16 );
			
			//a rectangle
			var rect:Section = Section.createFromValues( Vector.<Number>( [ -10,2, 10,2, 10,-2, -10,-2 ] ) );
			
			//and now, the primitives revisited
			
			//a box is a Square section linearily extruded
			var box:PTFLoft = new PTFLoft( line, Section.createRegularSection( 50, 4 ) );
			shapeList.push( box.mesh );
			
			//a cylinder is like box, with more sides
			var cylinder:PTFLoft = new PTFLoft( line, Section.createRegularSection( 50, 32 ) );
			shapeList.push( cylinder.mesh );
			
			//a pyramid is a box scaled to 0 at one end
			var pyramid:PTFLoft = new PTFLoft( line, Section.createRegularSection( 50, 4 ) );
			pyramid.setScales( Vector.<Number>([1, 0]), Vector.<Number>([1, 0]) );
			pyramid.update();
			shapeList.push( pyramid.mesh );
			
			//a torus is a disc lathed around an axis
			//or 
			//a disc that follows a circular path
			var torus:PTFLoft = new PTFLoft( disc.path, Section.createRegularSection( 20, 16 ), false );
			shapeList.push( torus.mesh );
			
			
			//the moebius strip is a torus with a rectangular section on which we apply a 180° rotation
			var moebius:PTFLoft = new PTFLoft( disc.path, rect, false, BLUE );
			moebius.setRotations( Vector.<Number>([0, Math.PI]) );
			moebius.update();
			shapeList.push( moebius.mesh );
			
			
			//a spring is a disc that follows an helicoidal path
			var spring:PTFLoft = new PTFLoft( helix.path, Section.createRegularSection( 10, 16 ), true );
			shapeList.push( spring.mesh );
			
			//a knot is a disc extruded along a path
			var knot:PTFLoft = new PTFLoft( knot_path.path, Section.createRegularSection( 5, 3 ), false );
			shapeList.push( knot.mesh );
			
			knot_path.P = 8;
			knot = new PTFLoft( knot_path.path, Section.createRegularSection( 5, 3 ), false );
			shapeList.push( knot.mesh );
			
			knot_path.P = 2;
			knot_path.Q = 1;
			knot = new PTFLoft( knot_path.path, Section.createRegularSection( 5, 3 ), false );
			shapeList.push( knot.mesh );
			
			
			///batch adding things to scene
			
			var colors:Array = [ RED, GREEN, BLUE, YELLOW, UV_DEBUG ];
			//colors = [ UV_DEBUG ];
			var i:int = 0;
			var margin:Number = 50;
			var count:int = Math.sqrt( shapeList.length );
			var size:int = ( count * objectSize ) + ( ( count - 1 ) *  margin );
			for ( i = 0; i < shapeList.length; i++) 
			{
				
				var obj:Object3D = shapeList[ i ];
				obj.x = -size / 2 + ( objectSize + margin ) * ( i % count );
				obj.y = size / 2 - ( objectSize  + margin ) * int( i / count );
				( obj as Mesh ).material = colors[ i % colors.length ];
				addMesh( obj );
				
			}
			spring.mesh.y -= objectSize * .5;
		}
		
	}

}