package geometry.loft 
{
	import away3d.entities.Mesh;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	import geometry.DrawPanel;
	import triga.spline.curves.Cubic;
	import triga.spline.shapes.Loft;
	import triga.spline.shapes.Section;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class ParametricLoft extends BaseScene 
	{
		private var pathPanel:DrawPanel;
		private var sectionPanel:DrawPanel;
		
		private var section:Section;
		private var loft:Loft;
		private var mesh:Mesh;
		
		public function ParametricLoft() 
		{
			setup();
			
			pathPanel = new DrawPanel( "path", 300, 600, 30 );
			pathPanel.x = pathPanel.y = 10;
			addChild( pathPanel );
			
			pathPanel.addEventListener( DrawPanel.CHANGE, onChange );
			pathPanel.addEventListener( DrawPanel.UPDATE, onPathUpdate );
			
			
			
			sectionPanel = new DrawPanel( "section", 300, 300, 10 );
			sectionPanel.x = stage.stageWidth - 310;
			sectionPanel.y = 10;
			addChild( sectionPanel );
			
			sectionPanel.addEventListener( DrawPanel.CHANGE, onChange );
			sectionPanel.addEventListener( DrawPanel.UPDATE, onChange );
			
		}
		
		private function onPathUpdate(e:Event):void 
		{
			
			var points:Vector.<Point> = pathPanel.points.concat();
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
			for each (var p:Point in points) 
			{
				path.push( new Vector3D( p.x, -p.y, 0 ) );
			}
			loft.path = new Cubic().compute( path );
			
		}
		
		private function onChange(e:Event):void 
		{
			
			compute();
			
		}
		
		private function compute():void 
		{
			
			if ( pathPanel.points.length < 1 ) return;
			if ( sectionPanel.points.length < 2 ) return;
			
			
			section = new Section( sectionPanel.points.concat() );
			
			var points:Vector.<Point> = pathPanel.points.concat();
			var path:Vector.<Vector3D> = new Vector.<Vector3D>();
			for each (var p:Point in points) 
			{
				path.push( new Vector3D( p.x, -p.y, 0 ) );
			}
			
			path = new Cubic().compute( path );
			
			
			loft = new Loft( path, section, true, UV_DEBUG );
			
			if (mesh != null )
			{
				removeMesh( mesh );
				mesh = null;
			}
			
			mesh = loft.mesh;
			addMesh( mesh );
			
		}
		
	}

}