package materials 
{
	import away3d.cameras.lenses.FreeMatrixLens;
	import away3d.lights.PointLight;
	import away3d.materials.BitmapMaterial;
	import away3d.materials.methods.BasicDiffuseMethod;
	import away3d.materials.methods.BasicSpecularMethod;
	import away3d.materials.methods.CelDiffuseMethod;
	import away3d.materials.methods.CelSpecularMethod;
	import away3d.materials.methods.FresnelSpecularMethod;
	import away3d.materials.methods.OutlineMethod;
	import away3d.materials.methods.RimLightMethod;
	import away3d.primitives.Sphere;
	import com.bit101.components.ColorChooser;
	import com.bit101.components.HUISlider;
	import com.bit101.components.Label;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.geom.Vector3D;
	import triga.render.MaterialHelper;
	/**
	 * @author Nicolas Barradeau
	 * http://en.nicoptere.net
	 */
	public class C_Bitmapmaterial extends BaseScene
	{
		
		[Embed(source = "../../lib/res/textures/UV_helper_1024.png")]
		private var textureSource:Class;
		private var texture:BitmapData = new textureSource().bitmapData;
		
		[Embed(source = "../../lib/res/textures/cubemap_512.png")]
		private var envSource:Class;
		private var envMap:BitmapData = new envSource().bitmapData;
		
		private var bitmapMaterial:BitmapMaterial;
		private var sphere:Sphere;
		
		/*****************
			
			UI
			
		*****************/
		private var diffuseColor:ColorChooser;
		private var glossiness:HUISlider;
		private var specularity:HUISlider;
		private var specularColor:ColorChooser;
		private var ambient:HUISlider;
		private var ambientColor:ColorChooser;
		private var light:PointLight;
		private var lightX:HUISlider;
		private var lightY:HUISlider;
		
		
		public function C_Bitmapmaterial() 
		{
			setup();
			
			//crée un materiau couleur aplat
			bitmapMaterial = new BitmapMaterial( texture, true, false, true );
			
			//crée un volume pour voir ce qu'on fait...
			sphere = new Sphere( bitmapMaterial, 200,32,24 );
			addMesh( sphere );
			
			
			//si on a une lumière, le matériau va renvoyer une partie de la lumière
			
			//associe un tableau d'objets lumière
			light = new PointLight();
			light.color = 0xFFFFFF;
			light.position = new Vector3D( 250, 250, -250 );
			addMesh( light );
			
			bitmapMaterial.lights = [ light ];
			
			
			/*
			gloss ( 1 - 100 ): force avec laquelle la lumière est renvoyée par le matériau
			*/
			bitmapMaterial.gloss = 1;
			
				//peut varier selon un map
				//colorMat.glossMap = bitmapdata;
			
			
			/*
			shininess ( 0 - 1 ): quantité de lumière renvoyée par le matériau 
			*/
			bitmapMaterial.specular = .5;
			
				//teinte de la lumière spéculaire
				bitmapMaterial.specularColor = 0xFFFFFF;
				
				//peut varier selon un map
				//colorMaterial.specularMap = bitmapdata;
				
			//methodes additionnelles
			
			//cell shading
			/*
			bitmapMaterial.diffuseMethod = new CelDiffuseMethod( 5 );
			bitmapMaterial.specularMethod = new CelSpecularMethod( .25 );
			//*/
			
			//outline (halo solide)
			//bitmapMaterial.addMethod( new OutlineMethod( 0xFFFFFF, 10 ) ) ;
			
			//rayonnement interne
			//bitmapMaterial.addMethod( new RimLightMethod( 0xFFFFFF ) );
			
			//joli mais bypass tous les réglages 
			//bitmapMaterial.specularMethod = new FresnelSpecularMethod( true );	
			
			
			//+
			MaterialHelper.createEnvrionmentMap( bitmapMaterial, envMap );
			
			
			/********************************************************************************
			
				UI
			
			********************************************************************************/
			var ox:int = 10;
			var oy:int = -10;
			
			//var diffuseColorLabel:Label = new Label( this, ox, oy += 20, 'difuse color' );
			//diffuseColor = new ColorChooser( this, ox + diffuseColorLabel.width + 10, oy, bitmapMaterial.color, onChange );
			
			specularity = new HUISlider( this, ox, oy += 20, 'specularity', onChange );
			specularity.value = bitmapMaterial.specular * 100;
			
			glossiness = new HUISlider( this, ox, oy += 20, 'glossiness', onChange );
			glossiness.minimum = 1;
			glossiness.value = bitmapMaterial.gloss;
			
			var specularColorLabel:Label = new Label( this, ox, oy += 20, 'specular color' );
			specularColor = new ColorChooser( this, ox + specularColorLabel.width +10, oy, 0xFFFFFF, onChange );
			
			ambient = new HUISlider( this, ox, oy += 20, 'ambient', onChange );
			ambient.value = 0;
			var ambientColorLabel:Label = new Label( this, ox, oy += 20, 'ambient color' );
			ambientColor = new ColorChooser( this, ox + specularColorLabel.width +10, oy, 0, onChange );
			
			oy += 40;
			lightX = new HUISlider( this, ox, oy += 20, 'light X', onChange );
			lightY = new HUISlider( this, ox, oy += 20, 'light Y', onChange );
			lightX.minimum = lightY.minimum = -300;
			lightX.maximum = lightY.maximum = 300;
			
			onChange();
			
		}
		
		private function onChange( e:Event = null ):void 
		{
			
			//bitmapMaterial.color = diffuseColor.value;
			bitmapMaterial.gloss = glossiness.value;
			bitmapMaterial.specular = specularity.value * .01;
			bitmapMaterial.specularColor = specularColor.value;
			bitmapMaterial.ambientColor = ambientColor.value;
			bitmapMaterial.ambient = ambient.value * .01;
			
			light.x = lightX.value;
			light.y = lightY.value;
			
		}
	}

}